/*****************************************************************/
/*                                                               */
/*   LifeGame                                                    */
/*                                                               */
/*   Auteur       LePhenixNoir                                   */
/*   Date         10.06.2014                                     */
/*   Description  A simple simulator of Conway's game of life    */
/*                                                               */
/*   Copyright (c) 2006 CASIO COMPUTER CO., LTD.                 */
/*                                                               */
/*****************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include <EasyInput.h>
#include <PixEdit.h>
#include "fxlib.h"

void Menu   (void);
void Config (int *, int *, unsigned char *);
int  getNum (void);
void Launch (int,int,unsigned char *);

int AddIn_main(int isAppli, unsigned short OptionNum)
{
  unsigned char field[1024] = { 0 };
  int sbs=0, max=100;

  PixEdit_SetImage(field);
  Menu();
  Config(&sbs,&max,field);
  Launch(sbs,max,field);

  return 1;
}

void Menu()
{
  const unsigned char title[348] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 48, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 24, 4, 0, 0, 0, 0, 120, 0, 0, 0, 0, 0, 30, 14, 15, 128, 0, 1, 252, 0, 0, 0, 0, 0, 30, 14, 31, 0, 0, 1, 254, 0, 0, 0, 0, 0, 30, 14, 31, 0, 0, 3, 222, 0, 0, 0, 0, 0, 30, 14, 28, 0, 0, 3, 216, 0, 8, 0, 0, 0, 30, 2, 28, 0, 0, 3, 192, 0, 4, 0, 0, 0, 30, 8, 60, 7, 192, 3, 192, 0, 4, 49, 129, 240, 30, 12, 63, 143, 224, 3, 222, 0, 15, 243, 195, 248, 30, 14, 63, 15, 240, 3, 222, 31, 207, 255, 195, 252, 30, 14, 63, 14, 240, 3, 158, 63, 207, 255, 195, 188, 30, 14, 62, 30, 240, 3, 222, 127, 207, 121, 199, 188, 30, 6, 60, 31, 240, 3, 158, 123, 199, 121, 199, 252, 30, 6, 60, 31, 240, 3, 142, 123, 207, 57, 199, 252, 30, 14, 28, 31, 128, 3, 222, 123, 199, 57, 199, 224, 30, 14, 28, 14, 16, 3, 222, 123, 207, 57, 195, 132, 31, 238, 28, 14, 176, 3, 254, 123, 207, 57, 195, 172, 31, 198, 28, 14, 240, 1, 254, 127, 199, 57, 195, 188, 31, 199, 28, 15, 224, 1, 254, 127, 199, 56, 195, 248, 32, 3, 28, 7, 224, 0, 254, 61, 195, 24, 225, 248, 0, 1, 12, 0, 128, 0, 15, 49, 192, 132, 32, 32, 0, 0, 6, 0, 0, 0, 1, 0, 64, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 32, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
  const unsigned char press[99] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 124, 249, 249, 227, 192, 243, 54, 97, 219, 246, 96, 50, 100, 203, 54, 97, 155, 54, 96, 241, 150, 96, 50, 100, 195, 6, 1, 155, 179, 192, 225, 131, 192, 60, 120, 241, 227, 193, 155, 241, 128, 193, 225, 128, 48, 112, 192, 48, 97, 251, 113, 128, 225, 129, 128, 48, 104, 203, 54, 97, 155, 49, 128, 241, 145, 128, 120, 237, 249, 227, 193, 155, 51, 193, 219, 243, 192, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
  const unsigned char dna[22] = { 40, 16, 40, 124, 68, 254, 130, 254, 130, 124, 68, 56, 16, 56, 68, 124, 130, 254, 130, 254, 68, 124 };
  extern unsigned char *Disp_VRAM;
  unsigned int key;
  int c=0,f=0, i,n;

  Disp_Init();
  Disp_Clear();
  Disp_Bitmap(title,32,2,96,29,1);
  Disp_Bitmap(press,38,39,84,9,1);
  PrintMini(59,50,(const unsigned char *)"LePhenixNoir",0);

  while(1)
  {
    for(n=0;n<4;n++) for(i=0;i<64;i++) *(Disp_VRAM+(i<<4)+n) = dna[(i+(c*(5-n)))%22];
    if(c++==22) c=0;
    if(f++>5) f=5;

    if(f==5 && GetKeyWait(KEYWAIT_HALTOFF_TIMEROFF,-1,1,&key)==KEYREP_KEYEVENT) return;
    Bdisp_PutDisp_DD();
    Sleep(40);
  }
}

void Config(int *sbs, int *max, unsigned char *field)
{
  unsigned int key;
  char ch[20];
  int i,p=0;

  Disp_Clear();
  for(i=0;i<127;i++) Disp_Pixel(i,3,1);
  PrintXY(19,0,(const unsigned char *)" Configuration ",0);
  locate(3,3); Print((const unsigned char *)"Step by step");
  locate(3,4); Print((const unsigned char *)"Generations");
  locate(3,5); Print((const unsigned char *)"Configure field");
  locate(3,7); Print((const unsigned char *)"Lauch game");
  SaveDisp(2);

  Sleep(100);
  key = 0;

  while(1)
  {
    RestoreDisp(2);
    locate(17,3); Print((const unsigned char *)(*sbs?"Yes":"No"));
    sprintf(ch,"%d",*max); locate(17,4); Print((const unsigned char *)ch);
    locate(2,p+3+(p==3)); Print((const unsigned char *)"\346\233");

    while(1)
    {
      GetKey(&key);

      if(key==30004) {
        if(!p) *sbs = !*sbs;
        else if (p==1) *max = getNum();
        else if (p==2) PixEdit_Edit();
        else if (p==3) return;
        break; }

      if(key==30018 && p)   { p--; break; }
      if(key==30023 && p-3) { p++; break; }
    }
  }
}

int getNum()
{
  char ch[5];

  PopUpWin(2);
  locate(3,3); Print((const unsigned char *)"Generations");
  locate(3,4); Print((const unsigned char *)"[    ]");
  EI_input_string(ch,4,ALLOW_09,4,4,8,LEFT,NORMAL);

  return atoi(ch);
}

void Launch(int sbs, int max, unsigned char *field)
{
  extern unsigned char *Disp_VRAM;
  unsigned int key;
  int a,b, x,y;
  int i,v, n,o;

  Bkey_Set_RepeatTime(5,2);
  for(v=0;v<1024;v++) *(Disp_VRAM+v) = *(field+v);

  for(i=1;i<max;i++)
  {
    for(v=0;v<1024;v++) *(field+v) = *(Disp_VRAM+v);

    if(sbs) {
      char ch[20];
      sprintf(ch,"Generation %d",i);
      locate(1,8); Print((const unsigned char *)ch);
      GetKey(&key); }

    else Bdisp_PutDisp_DD();

    for(v=0;v<1024;v++) *(Disp_VRAM+v) = *(field+v);

    for(y=1;y<63;y++) for(x=1;x<127;x++)
    {
      n = 0;
      o = PixEdit_GetPixel(x,y);
      for(a=-1;a<=1;a++) for(b=-1;b<=1;b++) if(a || b) n+=PixEdit_GetPixel(x+a,y+b);
      if(!o && n==3) Disp_Pixel(x,y,1);
      if(o && (n!=2&&n!=3)) Disp_Pixel(x,y,0);
    }
  }

  if(!sbs) GetKey(&key);

  Bkey_Set_RepeatTime_Default();
}

#pragma section _BR_Size
unsigned long BR_Size;
#pragma section
#pragma section _TOP
int InitializeSystem(int isAppli, unsigned short OptionNum) {
    return INIT_ADDIN_APPLICATION(isAppli, OptionNum); }
#pragma section
/* AddIn_main(isAppli(1=MAIN,0=eAct), OptionNum(isAppli==0?0~3)) :: int(Done?1:0) */


